"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const qwik = require("@builder.io/qwik");
const jsxRuntime = require("@builder.io/qwik/jsx-runtime");
const sortOrder = {
  NONE: 0,
  ASCENDING: 1,
  DESENDING: 2
};
const filterType = {
  SEARCH: "search",
  OPTIONS: "options"
};
const arrowConfig = {
  size: 24,
  defaultColor: "#ccc",
  highlightColor: "#fff"
};
const defaultClassNames = {
  table: "table",
  thead: "table-head",
  th: "table-heading",
  thContent: "table-heading-content",
  thText: "table-heading-text",
  sortContainer: `sort-container size-[${arrowConfig.size}] default-[${arrowConfig.defaultColor}] highlight-[${arrowConfig.highlightColor}]`,
  sortArrowUp: "sort-arrow-up",
  sortArrowDown: "sort-arrow-down",
  filterContainer: "filter-container",
  filterInput: "filter-input",
  tbody: "table-body",
  tr: "table-row",
  td: "table-cell"
};
const SvgArrowUp = (props, key) => {
  return /* @__PURE__ */ qwik.createElement("svg", {
    xmlns: "http://www.w3.org/2000/svg",
    width: "1em",
    height: "1em",
    viewBox: "0 0 24 24",
    ...props,
    key
  }, /* @__PURE__ */ qwik._jsxQ("path", null, {
    fill: "currentColor",
    d: "m7 14l5-5l5 5H7Z"
  }, null, 3, "ep_0"));
};
const SvgArrowDown = (props, key) => {
  return /* @__PURE__ */ qwik.createElement("svg", {
    xmlns: "http://www.w3.org/2000/svg",
    width: "1em",
    height: "1em",
    viewBox: "0 0 24 24",
    ...props,
    key
  }, /* @__PURE__ */ qwik._jsxQ("path", null, {
    fill: "currentColor",
    d: "m12 15l-5-5h10l-5 5Z"
  }, null, 3, "ep_1"));
};
const SortArrows = /* @__PURE__ */ qwik.componentQrl(/* @__PURE__ */ qwik.inlinedQrl((props) => {
  let size = arrowConfig.size;
  let defaultColor = arrowConfig.defaultColor;
  let highlightColor = arrowConfig.highlightColor;
  props.classNames.sortContainer?.split(" ").map((c) => {
    if (c.includes("size-["))
      size = Number(c.substring(6, c.length - 1));
    else if (c.includes("default-["))
      defaultColor = c.substring(9, c.length - 1);
    else if (c.includes("highlight-["))
      highlightColor = c.substring(11, c.length - 1);
  });
  const handleClick = /* @__PURE__ */ qwik.inlinedQrl(() => {
    const [props2] = qwik.useLexicalScope();
    if (props2.sortConfig.param === props2.heading)
      props2.sortConfig.order = (props2.sortConfig.order + 1) % 3;
    else {
      props2.sortConfig.param = props2.heading;
      props2.sortConfig.order = sortOrder.ASCENDING;
    }
  }, "SortArrows_component_handleClick_1C3t6DWVBzU", [
    props
  ]);
  return /* @__PURE__ */ qwik._jsxQ("div", null, {
    class: qwik._fnSignal((p0) => p0.classNames.sortContainer, [
      props
    ], "p0.classNames.sortContainer"),
    onClick$: handleClick
  }, [
    /* @__PURE__ */ qwik._jsxC(SvgArrowUp, {
      get class() {
        return props.classNames.sortArrowUp;
      },
      width: size,
      height: size,
      color: props.sortConfig.param === props.heading && props.sortConfig.order === sortOrder.ASCENDING ? highlightColor : defaultColor,
      [qwik._IMMUTABLE]: {
        class: qwik._fnSignal((p0) => p0.classNames.sortArrowUp, [
          props
        ], "p0.classNames.sortArrowUp")
      }
    }, 3, "ep_2"),
    /* @__PURE__ */ qwik._jsxC(SvgArrowDown, {
      get class() {
        return props.classNames.sortArrowDown;
      },
      width: size,
      height: size,
      color: props.sortConfig.param === props.heading && props.sortConfig.order === sortOrder.DESENDING ? highlightColor : defaultColor,
      [qwik._IMMUTABLE]: {
        class: qwik._fnSignal((p0) => p0.classNames.sortArrowDown, [
          props
        ], "p0.classNames.sortArrowDown")
      }
    }, 3, "ep_3")
  ], 1, "ep_4");
}, "SortArrows_component_ycNryhVjMpU"));
const FilterInput = /* @__PURE__ */ qwik.componentQrl(/* @__PURE__ */ qwik.inlinedQrl((props) => {
  const handleFilter = /* @__PURE__ */ qwik.inlinedQrl(
    // eslint-disable-next-line
    (event) => {
      const [props2] = qwik.useLexicalScope();
      return props2.filterConfig.params = {
        ...props2.filterConfig.params,
        [props2.heading]: event?.target?.value
      };
    },
    "FilterInput_component_handleFilter_Y7tA6sBt0Fk",
    [
      props
    ]
  );
  switch (props.filterType) {
    case filterType.SEARCH:
      return /* @__PURE__ */ qwik._jsxQ("input", null, {
        type: "text",
        class: qwik._fnSignal((p0) => p0.classNames.filterInput, [
          props
        ], "p0.classNames.filterInput"),
        onInput$: handleFilter
      }, null, 3, "C2_0");
    case filterType.OPTIONS:
      return /* @__PURE__ */ qwik._jsxQ("select", null, {
        name: qwik._fnSignal((p0) => p0.heading, [
          props
        ], "p0.heading"),
        id: qwik._fnSignal((p0) => p0.heading, [
          props
        ], "p0.heading"),
        class: qwik._fnSignal((p0) => p0.classNames.filterInput, [
          props
        ], "p0.classNames.filterInput"),
        value: "---",
        onChange$: handleFilter
      }, [
        /* @__PURE__ */ qwik._jsxQ("option", null, {
          value: "---"
        }, "---", 3, null),
        props.options?.map((option, index) => {
          return /* @__PURE__ */ qwik._jsxQ("option", {
            value: option?.key?.toString()
          }, null, option?.value?.toString(), 1, index);
        })
      ], 1, null);
    default:
      return /* @__PURE__ */ qwik._jsxC(jsxRuntime.Fragment, null, 3, "C2_1");
  }
}, "FilterInput_component_YrdchsjlZKA"));
const param2Text = /* @__PURE__ */ qwik.inlinedQrl((param) => {
  if (param === "id")
    return "ID";
  let returnVal = "";
  const camelCaseWords = param.replace(/([A-Z])/g, " $1");
  const result = camelCaseWords.charAt(0).toUpperCase() + camelCaseWords.slice(1);
  const snakeCaseWords = result.split(/_|-/);
  snakeCaseWords.forEach((word, index) => returnVal += word.charAt(0).toUpperCase() + word.slice(1) + (index === snakeCaseWords.length - 1 ? "" : " "));
  return returnVal;
}, "param2Text_agI06SZVpyY");
const value2Options = /* @__PURE__ */ qwik.inlinedQrl((data, param) => {
  const options = [];
  data.map((record) => {
    const option = {
      // TODO: type-scrict these lines
      key: record[param],
      value: record[param]
    };
    options.push(option);
  });
  return options;
}, "value2Options_DpmJJ1Z1KpY");
const TableHeader = /* @__PURE__ */ qwik.componentQrl(/* @__PURE__ */ qwik.inlinedQrl((props) => {
  return /* @__PURE__ */ qwik._jsxQ("thead", null, {
    class: qwik._fnSignal((p0) => p0.classNames.thead, [
      props
    ], "p0.classNames.thead")
  }, /* @__PURE__ */ qwik._jsxQ("tr", null, null, props.headings.map((heading) => /* @__PURE__ */ qwik._jsxBranch(/* @__PURE__ */ qwik._jsxQ("th", {
    class: `${props.classNames.th}${props.classNames.tcol?.[heading] ? " " + props.classNames.tcol?.[heading] : ""}`
  }, null, /* @__PURE__ */ qwik._jsxQ("div", null, {
    class: qwik._fnSignal((p0) => p0.classNames.thContent, [
      props
    ], "p0.classNames.thContent")
  }, [
    /* @__PURE__ */ qwik._jsxQ("div", null, {
      class: qwik._fnSignal((p0) => p0.classNames.thText, [
        props
      ], "p0.classNames.thText")
    }, [
      props.customHeadings?.[heading] || param2Text(heading),
      props.sortOptions?.params.includes(heading) ? /* @__PURE__ */ qwik._jsxC(SortArrows, {
        heading,
        get classNames() {
          return props.classNames;
        },
        get sortConfig() {
          return props.sortConfig;
        },
        [qwik._IMMUTABLE]: {
          classNames: qwik._fnSignal((p0) => p0.classNames, [
            props
          ], "p0.classNames"),
          sortConfig: qwik._fnSignal((p0) => p0.sortConfig, [
            props
          ], "p0.sortConfig")
        }
      }, 3, "Lj_0") : /* @__PURE__ */ qwik._jsxC(jsxRuntime.Fragment, null, 3, "Lj_1")
    ], 1, null),
    props.filterOptions?.params ? Object.keys(props.filterOptions?.params).includes(heading) ? /* @__PURE__ */ qwik._jsxQ("div", null, {
      class: qwik._fnSignal((p0) => p0.classNames.filterContainer, [
        props
      ], "p0.classNames.filterContainer")
    }, /* @__PURE__ */ qwik._jsxC(FilterInput, {
      heading,
      get classNames() {
        return props.classNames;
      },
      filterType: props.filterOptions?.params?.[heading],
      get filterConfig() {
        return props.filterConfig;
      },
      options: props.filterOptions.options?.[heading],
      [qwik._IMMUTABLE]: {
        classNames: qwik._fnSignal((p0) => p0.classNames, [
          props
        ], "p0.classNames"),
        filterConfig: qwik._fnSignal((p0) => p0.filterConfig, [
          props
        ], "p0.filterConfig")
      }
    }, 3, "Lj_2"), 1, "Lj_3") : /* @__PURE__ */ qwik._jsxQ("div", null, {
      class: qwik._fnSignal((p0) => p0.classNames.filterContainer, [
        props
      ], "p0.classNames.filterContainer"),
      style: "border: solid 1px yellow"
    }, null, 3, null) : /* @__PURE__ */ qwik._jsxC(jsxRuntime.Fragment, null, 3, "Lj_4")
  ], 1, null), 1, heading))), 1, null), 1, "Lj_5");
}, "TableHeader_component_oOJ0YwcCTJc"));
const TableBody = /* @__PURE__ */ qwik.componentQrl(/* @__PURE__ */ qwik.inlinedQrl((props) => {
  return /* @__PURE__ */ qwik._jsxQ("tbody", null, {
    class: qwik._fnSignal((p0) => p0.classNames.tbody, [
      props
    ], "p0.classNames.tbody")
  }, props.data.map((record, index) => {
    return /* @__PURE__ */ qwik._jsxQ("tr", null, {
      class: qwik._fnSignal((p0) => p0.classNames.tr, [
        props
      ], "p0.classNames.tr")
    }, props.headings.map((heading) => /* @__PURE__ */ qwik._jsxQ("td", {
      class: `${props.classNames.td}${props.classNames.tcol?.[heading] ? " " + props.classNames.tcol?.[heading] : ""}`
    }, null, props.element?.[heading]?.(record, heading) || record[heading], 1, heading)), 1, index);
  }), 1, "Oe_0");
}, "TableBody_component_KzgCLC9bSzA"));
const sortTable = /* @__PURE__ */ qwik.inlinedQrl((data, param, isAscending) => {
  const sortedData = isAscending ? [
    ...data
  ].sort((a, b) => {
    if (a[param] < b[param])
      return -1;
    if (a[param] > b[param])
      return 1;
    return 0;
  }) : [
    ...data
  ].sort((a, b) => {
    if (b[param] < a[param])
      return -1;
    if (b[param] > a[param])
      return 1;
    return 0;
  });
  return sortedData;
}, "sortTable_boqfopb4j58");
const getTotalPages = /* @__PURE__ */ qwik.inlinedQrl((total_record, record_per_page) => {
  return Math.floor(total_record / record_per_page) + (total_record % record_per_page === 0 ? 0 : 1);
}, "getTotalPages_Tyqy0V5HWeQ");
const TableLoader = /* @__PURE__ */ qwik.componentQrl(/* @__PURE__ */ qwik.inlinedQrl((props) => {
  const table = qwik.useStore({
    data: props.tData,
    sortConfig: {
      param: "",
      order: 0
    },
    filterConfig: {
      params: {}
    }
  });
  const headings = Object.keys(props.tData[0]);
  const theadClassNames = {
    thead: props.tableOptions?.classNames?.thead || defaultClassNames.thead,
    th: props.tableOptions?.classNames?.th || defaultClassNames.th,
    thContent: props.tableOptions?.classNames?.thContent || defaultClassNames.thContent,
    thText: props.tableOptions?.classNames?.thText || defaultClassNames.thText,
    sortContainer: props.tableOptions?.classNames?.sortContainer || defaultClassNames.sortContainer,
    sortArrowUp: props.tableOptions?.classNames?.sortArrowUp || defaultClassNames.sortArrowUp,
    sortArrowDown: props.tableOptions?.classNames?.sortArrowDown || defaultClassNames.sortArrowDown,
    filterContainer: props.tableOptions?.classNames?.filterContainer || defaultClassNames.filterContainer,
    filterInput: props.tableOptions?.classNames?.filterInput || defaultClassNames.filterInput,
    td: props.tableOptions?.classNames?.td || defaultClassNames.td,
    tcol: props.tableOptions?.classNames?.tcol || void 0
  };
  const tbodyClassNames = {
    tbody: props.tableOptions?.classNames?.tbody || defaultClassNames.tbody,
    tr: props.tableOptions?.classNames?.tr || defaultClassNames.tr,
    td: props.tableOptions?.classNames?.td || defaultClassNames.td,
    tcol: props.tableOptions?.classNames?.tcol || void 0
  };
  qwik.useTaskQrl(/* @__PURE__ */ qwik.inlinedQrl(async ({ track }) => {
    const [headings2, props2, table2] = qwik.useLexicalScope();
    track(() => [
      table2.sortConfig.param,
      table2.sortConfig.order,
      table2.filterConfig.params
    ]);
    const filteredData = props2.tData.filter((record) => {
      let i = 0;
      headings2.map((id) => {
        if (props2.tableOptions?.filterOptions?.params?.[id] === filterType.SEARCH) {
          if (record[id]?.toString().toLowerCase().includes(table2.filterConfig.params[id]?.toString().toLowerCase()))
            ++i;
        } else if (props2.tableOptions?.filterOptions?.params?.[id] === filterType.OPTIONS) {
          if (record[id]?.toString() === table2.filterConfig.params[id]?.toString() || table2.filterConfig.params[id] === "---")
            ++i;
        }
      });
      if (i === Object.keys(table2.filterConfig.params).length)
        return record;
    });
    if (table2.sortConfig.order === sortOrder.NONE)
      table2.data = filteredData;
    else
      table2.data = await sortTable(filteredData, table2.sortConfig.param, table2.sortConfig.order === sortOrder.ASCENDING);
  }, "TableLoader_component_useTask_kEndS0EIkNo", [
    headings,
    props,
    table
  ]));
  return /* @__PURE__ */ qwik._jsxQ("table", {
    class: props.tableOptions?.classNames?.table || defaultClassNames.table
  }, null, [
    /* @__PURE__ */ qwik._jsxC(TableHeader, {
      headings,
      get customHeadings() {
        return props.tableOptions?.customHeadings;
      },
      classNames: theadClassNames,
      get sortOptions() {
        return props.tableOptions?.sortOptions;
      },
      get sortConfig() {
        return table.sortConfig;
      },
      get filterOptions() {
        return props.tableOptions?.filterOptions;
      },
      get filterConfig() {
        return table.filterConfig;
      },
      [qwik._IMMUTABLE]: {
        customHeadings: qwik._fnSignal((p0) => p0.tableOptions?.customHeadings, [
          props
        ], "p0.tableOptions?.customHeadings"),
        sortOptions: qwik._fnSignal((p0) => p0.tableOptions?.sortOptions, [
          props
        ], "p0.tableOptions?.sortOptions"),
        sortConfig: qwik._fnSignal((p0) => p0.sortConfig, [
          table
        ], "p0.sortConfig"),
        filterOptions: qwik._fnSignal((p0) => p0.tableOptions?.filterOptions, [
          props
        ], "p0.tableOptions?.filterOptions"),
        filterConfig: qwik._fnSignal((p0) => p0.filterConfig, [
          table
        ], "p0.filterConfig")
      }
    }, 3, "h1_0"),
    /* @__PURE__ */ qwik._jsxC(TableBody, {
      get data() {
        return table.data;
      },
      headings,
      classNames: tbodyClassNames,
      get element() {
        return props.element;
      },
      [qwik._IMMUTABLE]: {
        data: qwik._fnSignal((p0) => p0.data, [
          table
        ], "p0.data"),
        element: qwik._fnSignal((p0) => p0.element, [
          props
        ], "p0.element")
      }
    }, 3, "h1_1")
  ], 1, "h1_2");
}, "TableLoader_component_hOQfsSkMQGw"));
exports.default = TableLoader;
exports.filterType = filterType;
exports.getTotalPages = getTotalPages;
exports.value2Options = value2Options;
